### Code for Castro Sanchez-Bermejo et al., 2023 "Within-Individual Leaf Trait 
# Variation Increases with Phenotypic Integration in a Subtropical Tree Diversity 
# Experiment ###

# Libraries

library(readxl) # For reading the datasets
library(tidyverse) # For data management
library(dplyr) # For data management
library(FD) # "gowdis" and "dbFD" functions to assess the functional indices
library(matrixStats) # "rowSds" function to assess the Standarized Effect Sizes
library(jmuOutlier) # "perm.cor.test" function to assess the number of significant correlations between traits
library(igraph) # "graph.data.frame" function to build trait networks based in trait correlations. "degree" and "edge_density" functions to assess phenotypic integration properties 

# Reading the data
# We use two datasets: (1) The leaf-level dataset, which contains information of functional traits
# at leaf level + info about the diversity of the TSP, Tree ID, TSP ID, Plot and Site, 
# and (2) the neighbor dataset, which contains the information of the local neighborhood
# of our trees.

data <- read_excel("BEF-China.xlsx", sheet = "Leaf traits") # Leaf-level dataset
neighbourhood <- read_excel("BEF-China.xlsx", sheet = "Local neighborhood") # Neighborhood dataset

#############################################################
#### Taxonomic diversity of the surrounding neighborhood ####
#############################################################

# We calculate Shannon diversity of the local neighborhood for every TSP

trees <- data[!duplicated(data$TreeID), ] # We create a dataset for the tree-level that will be used in the analyses
trees <- trees [, 2:8]
shannon <- as.data.frame(cbind(neighbourhood$TreeID, vegan::diversity(neighbourhood[, -1]))) # We assess shannon diversity for the local neighborhood of every tree
names(shannon) <- c("TreeID", "shannon")
shannon$shannon <- as.numeric(shannon$shannon)
trees <- merge(trees, shannon, by = "TreeID") # We merge our shannon diversity and the tree level dataset

#########################
#### Trait variation ####
#########################

#### Measuring trait-based functional indices ####
# In order to measure the indices of functional diversity, we first measure the Gower distance between traits.
# Fist, we calculate gower distance using the function "gowdis".
# Then, based in the distance matrix, we use the function "dbFD", 
# which performs different functional indices including FRic and FDis.
# For the functional indices we used the correction method defined by Lingoes,
# that adds the minimum value for the eigenvalue in the PCoA in order to avoid
# the presence of negative eigenvalues.

FRic <- rep(NA, 499) # Empty vector to store the FRic data
FDis <- rep(NA, 499) # Empty vector to store the FDis data

for (i in 1:499) {
  presence <- subset(data, TreeID == trees[i, ]$TreeID) %>%
    mutate(present = 1) %>%
    pivot_wider(names_from = TreeID, values_from = present)  # Presence/absence matrix for a tree
  
  vector <- data.frame(t(presence[, 17]))
  names(vector) <- presence$LeafID              
  leaves.distance <- gowdis(presence[, 9:17]) # Leaf distance matrix 
  leaves.distance <- as.dist(as.matrix(leaves.distance))
  attr(leaves.distance, "Labels") <- names(vector)
  
  functional <- dbFD(leaves.distance, vector, 
                     calc.FGR = FALSE, calc.CWM = FALSE, calc.FDiv = FALSE,
                     corr = "lingoes",
                     messages = FALSE) # We assess the functional indices
  
  FRic[i] <- functional$FRic # We store the results for FRic
  FDis[i] <- functional$FDis # We store the results for FDis
}

#### Using Standardized effect sizes (SESs) for the standardization of indices #####
# As the functional indices that we are using are affected by the number of leaves
# per tree (specially FRic), we assess the standarized effect size (SESs) for every
# index for every tree. To do so, we will create 999 simulations in which we will assign
# random leaves from the total pools of leaves in our dataset to every individual. Thus,
# we maintain the number of leaves per tree but these are assigned randomly. Finally,
# we compare the simulated values by: SES = (observed - mean(simulated)/sd(simulated))

# We perform random simulations following 
# L?pez-Angulo, J., de la Cruz, M., Pescador, D. S., S?nchez, A. M., & Escudero, A. (2020).
# A dimmer shade of pale: revealing the faint signature of local assembly processes
# on the structure of strongly filtered plant communities. Ecography, 44(1), 87-97.
# (https://doi.org/10.1111/ecog.05405)

# We create a function to perform random simulations based on a probability matrix
# (from L?pez-Angulo et al. 2020)
simu.com <- function(abundance, prob, nsim) {
  stochasticness <- apply(abundance, 1, function(x) sum(x>0))
  nsp <- ncol(abundance)
  cat("\n\n", "simulating", nsim, "null assemblages", "\n\n")
  result <- list()
  for(i in 1: nsim){
    null.assemblage <- t(mapply(function(x, y) (1:nsp)%in%sort(sample(1:nsp, size = x, prob = y)),
                                stochasticness, data.frame(t(prob))))
    com.sim <- null.assemblage
    for (sp in 1:ncol(null.assemblage)){
      com.sim[null.assemblage[, sp] > 0, sp] <- sample(abundance[abundance[, sp] > 0, sp],
                                                       size = sum(null.assemblage[, sp] >0),
                                                       replace = TRUE)
    }
    colnames(com.sim) <- colnames(abundance)
    rownames(com.sim) <- rownames(abundance)
    result[[i]] <- com.sim
  }
  return(result)
} 

# We include a matrix of presence of leaves in every tree as a base for the randomizations
presence <- data %>%
  mutate(present = 1) %>%
  pivot_wider(names_from = TreeID, values_from = present)
presence <- data.frame(t(presence[, 17:515]))  
names(presence) <- data$LeafID
presence[is.na(presence)] <- 0

# We create our virtual randomizations

set.seed(0)
stochastic_prob <- matrix(data = sum(ifelse(presence > 0, 1, 0)) / nrow(presence) / ncol(presence),
                          nrow = nrow(presence),
                          ncol = ncol(presence)) # Probability matrix
set.seed(0)
stochastic_matrices <- simu.com(presence, stochastic_prob, nsim = 500) # List of randomizations

random.FRic <- matrix(NA, 499, 500) # We create an empty matrix to store the results of FRic from the randomizations
random.FDis <- matrix(NA, 499, 500) # We create an empty matrix to store the results of FDis from the randomizations

for (j in 1:500) {
  simulation <- stochastic_matrices[[j]]
  
  for (i in 1:499) {
    random <- as.data.frame(t(simulation)[, i])
    names(random) <- "LeafID"
    random <- as.data.frame(t(subset(random, LeafID != 0)))
    nombres <- names(random)
    # Based on the LeafID and TreeID, we create the presence/absence matrix
    presence <- subset(data, LeafID == names(random)[1]| LeafID == names(random)[2]| LeafID == names(random)[3]| LeafID == names(random)[4]|
                         LeafID == names(random)[5]| LeafID == names(random)[6]| LeafID == names(random)[7]| LeafID == names(random)[8]|
                         LeafID == names(random)[9]| LeafID == names(random)[10]| LeafID == names(random)[11]| LeafID == names(random)[12]|
                         LeafID == names(random)[13]| LeafID == names(random)[14]| LeafID == names(random)[15])%>%
      mutate(present = 1) 
    vector <- data.frame(t(presence[, 18]))
    names(vector) <- presence$LeafID              
    leaves.distance <- gowdis(presence[, 9:16]) # Measuring leaf distance matrix (gower distance)
    leaves.distance <- as.dist(as.matrix(leaves.distance))
    attr(leaves.distance, "Labels") <- names(vector)
    
    functional <- dbFD(leaves.distance, vector, 
                       calc.FGR = FALSE, calc.CWM = FALSE, calc.FDiv = FALSE,
                       corr = "lingoes",
                       messages = FALSE) # Assessing the functional indices
    
    random.FRic[i, j] <- functional$FRic
    random.FDis[i, j] <- functional$FDis
  }
  
  print(paste("randomization number", j, "out of 500"))
}

# We estimate SES for both indices

SES.FRic <- (FRic - rowMeans(random.FRic)) / rowSds(random.FRic)
SES.FDis <- (FDis - rowMeans(random.FDis)) / rowSds(random.FDis)

#### Assessing trait variation for every single trait ####
# We used sd to assess trait variation for every single trait

sd.sla <- aggregate(data$SLA, by = list(data$TreeID), FUN = sd)$x
sd.ldmc <- aggregate(data$LDMC, by = list(data$TreeID), FUN = sd)$x
sd.cn <- aggregate(data$CN, by = list(data$TreeID), FUN = sd)$x
sd.c <- aggregate(data$C, by = list(data$TreeID), FUN = sd)$x
sd.n <- aggregate(data$N, by = list(data$TreeID), FUN = sd)$x
sd.mg <- aggregate(data$MG, by = list(data$TreeID), FUN = sd)$x
sd.k <- aggregate(data$K, by = list(data$TreeID), FUN = sd)$x
sd.ca <- aggregate(data$CA, by = list(data$TreeID), FUN = sd)$x
sd.p <- aggregate(data$P, by = list(data$TreeID), FUN = sd)$x


###########################
#### Trait integration ####
###########################

#### Measuring trait network properties ####

# To estimate the edge density in the plant networks, first we perform all correlations
# of traits for every tree. Then, we perform tests based on permutations (with 20,000 permutations each)
# and build the plant trait network including only those significant correlations
# (p < 0.05) with correlation coefficient above 0.6.
# Finally, for we build the network and measure edge density and degree.

trait.properties <- matrix(NA, 499, 10) # We create an empty matrix to store the results of edge density and degree for every trait

for (i in 1:499) {
  significant.correlations <- matrix(NA, 9, 9)
  colnames(significant.correlations) <- colnames(data[, c(9:17)])
  rownames(significant.correlations) <- colnames(data[, c(9:17)])
  for (k in 1:9) {
    for (j in k:9) {
      set.seed(0)
      significant.correlations[k, j] <- perm.cor.test(subset(data, TreeID == trees$TreeID[i])[, 8 + k][[1]], 
                                                      subset(data, TreeID == trees$TreeID[i])[, 8 + j][[1]])$p.value
    }}
  
  correlations <- cor(subset(data, TreeID == trees$TreeID[i])[, 9:17])
  
  values <- c(significant.correlations[1, 1:9], significant.correlations[2, 2:9],
              significant.correlations[3, 3:9], significant.correlations[4, 4:9], 
              significant.correlations[5, 5:9], significant.correlations[6, 6:9], 
              significant.correlations[7, 7:9], significant.correlations[8, 8:9],
              significant.correlations[9, 9])
  
  r <- c(correlations[1, 1:9], correlations[2, 2:9],
         correlations[3, 3:9], correlations[4, 4:9], 
         correlations[5, 5:9], correlations[6, 6:9], 
         correlations[7, 7:9], correlations[8, 8:9],
         correlations[9, 9])
  
  Var1 <- c(rep(rownames(correlations)[1], 9), rep(rownames(correlations)[2], 8),
            rep(rownames(correlations)[3], 7), rep(rownames(correlations)[4], 6),
            rep(rownames(correlations)[5], 5), rep(rownames(correlations)[6], 4),
            rep(rownames(correlations)[7], 3), rep(rownames(correlations)[8], 2),
            rownames(correlations)[9])
  
  Var2 <- c(rownames(correlations)[1: 9], rownames(correlations)[2: 9],
            rownames(correlations)[3: 9], rownames(correlations)[4: 9],
            rownames(correlations)[5: 9], rownames(correlations)[6: 9],
            rownames(correlations)[7: 9], rownames(correlations)[8: 9],
            rownames(correlations)[9])
  
  cor <- data.frame(cbind(Var1, Var2, values, r))
  cor$values <- as.numeric(cor$values)
  cor$r <- as.numeric(cor$r)
  
  # Based on our selected correlations (the significant ones) we build our network
  my_adj_list <- cor %>% filter(values < 0.05)
  my_adj_list <- my_adj_list %>% filter(abs(r) > 0.6)
  names(my_adj_list) <- c("from", "to", "weight", "r") # We define the origin and end of our nodes
  net <- graph.data.frame(my_adj_list, directed = FALSE) # We build the network
  trait.properties[i, ] <- c(edge_density(net, loops = FALSE), degree(net)) # We store edge density and degree for every tree in our matrix
  print(paste("Tree number", i, "out of 499"))
}

trait.properties <- as.data.frame(trait.properties)
names(trait.properties) <- c("edge.density", "deg.sla", "deg.ldmc", "deg.cn",
                             "deg.c", "deg.n", "deg.mg", "deg.k", "deg.ca", "deg.p")

#### Storing all data ####
# We store all metrics of trait variation and phenotypic integration in a dataframe
# for further analyses

dataset <- cbind(trees, SES.FDis, SES.FRic, trait.properties,
                 sd.sla, sd.ldmc, sd.cn, sd.c, sd.n,
                 sd.mg, sd.k, sd.ca, sd.p)

#### Saving data (activate line) ####
# save(dataset, file = "Phen_int_BEFChina.RData") 